/*
**  SLOPE_REV: C-XSC classes for one- and multidimensional (interval)
**             slope arithmetic in reverse mode
**
**  Copyright (C) 1999-2001 M. Braeuer, W. Hofschuster, W. Kraemer
**                          Wiss. Rechnen/Softwaretechnologie
**                          Universitaet Wuppertal, Germany   
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Library General Public
**  License as published by the Free Software Foundation; either
**  version 2 of the License, or (at your option) any later version.
**
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
**  Library General Public License for more details.
**
**  You should have received a copy of the GNU Library General Public
**  License along with this library; if not, write to the Free
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "s_reverse.h"
#include <iostream>

using namespace cxsc;
using namespace std;

rslope f1(STAPE& t, const rslope_vec& x)
{
  //Alle vorkommenden Variablen muessen zu t gehoeren/initialisiert sein

 rslope erg;

 erg=_real(4.0)*sqr(x[1])-_interval(21)/_interval(10)*power(x[1],4)+
     _interval(1.0)/_interval(3.0)*power(x[1],6)+
     x[1]*x[2]-_real(4.0)*sqr(x[2])+_real(4.0)*power(x[2],4);

 return erg;
}


rslope_vec f2(STAPE& t, const rslope_vec& x)
{
  //Alle vorkommenden Variablen muessen zu t gehoeren/initialisiert sein

 rslope_vec erg(2);

 erg[1]=_real(4.0)*sqr(x[1])-_interval(21)/_interval(10)*power(x[1],4)+
        _interval(1.0)/_interval(3.0)*power(x[1],6)+
        x[1]*x[2]-_real(4.0)*sqr(x[2])+_real(4.0)*power(x[2],4);

 erg[2]=real(4.0)*sqr(x[1])-_interval(21)/_interval(10)*power(x[1],-4)+
        _interval(1.0)/_interval(3.0)*power(x[1],-6)+
        x[1]*x[2]-_real(4.0)*sqr(x[2])+_real(4.0)*power(x[2],-4);

 return erg;
}

int main(){

  /*----------------------------------------------------------------------

    Beispielprogramm fuer Anwendung der bereitgstellten Funktionen
    zur schnellen Berechnung von Steigungen und Steigungsvektoren

  ----------------------------------------------------------------------*/

 cout << endl;

 STAPE t1; //Tape auf das geschrieben wird

 ivector x_in(2); 
 x_in[1]=_interval(8,8.25);
 x_in[2]=_interval(3,3.25);

 ivector slopes(2);

 /*-----------------------------------------------------------------------
 Auswertung der Funktion f1
 -----------------------------------------------------------------------*/

 interval f_val, fc_val;

 forward_sweep(t1, f1, x_in, f_val, fc_val); //Entwicklungspunkte sind 
                                             //Mittelpkt.-e
 reverse_sweep(t1,slopes,1);

 cout << "Funktionswert: x " << f_val << endl;
 cout << "Funktionswert: c " << fc_val << endl;

 cout << "Slopes :" << endl;
 cout << slopes << endl;

 /*-----------------------------------------------------------------------
 Auswertung der Funktion f2
 -----------------------------------------------------------------------*/

 ivector f_valv(2), fc_valv(2);
 imatrix slopematrix(2,2);

 forward_sweep(t1, f2, x_in, f_valv, fc_valv); //Entwicklungspunkte sind
                                               // Mittelpkt.-e
 reverse_sweep(t1,slopematrix);

 cout << "Funktionswerte: x " << endl << f_valv << endl;
 cout << "Funktionswerte: c " << endl << fc_valv << endl;

 cout << "Slopematrix :" << endl;
 cout << slopematrix << endl;
}

/* 
gibt Funktionswerte der zweiten Komponente des Argumentsteigungstupels zurueck

Funktionswert: x [7.821482E+004,9.720767E+004]
Funktionswert: c [8.738000E+004,8.738001E+004]
Slopes :
[6.360685E+004,6.926493E+004]
[4.423828E+002,5.019922E+002]

Funktionswerte: x 
[7.821482E+004,9.720767E+004]
[2.377853E+002,2.631115E+002]

Funktionswerte: c 
[8.738000E+004,8.738001E+004]
[2.504320E+002,2.504321E+002]

Slopematrix :
[6.360685E+004,6.926493E+004]
[4.423828E+002,5.019922E+002]

[ 67.500217, 68.750258]
[-17.442105,-16.418211]

 */
